SystemOrganization addCategory: #SMLoader!

SystemWindow subclass: #SMLoader
	instanceVariableNames: 'packagesList selectedItemWrapper selectedCategoryWrapper filters categoriesToFilterIds'
	classVariableNames: 'DefaultCategoriesToFilterIds DefaultFilters'
	poolDictionaries: ''
	category: 'SMLoader'!

!SMLoader commentStamp: '<historical>' prior: 0!
A simple package loader that is currently the standard UI for SqueakMap (the model is an SMSqueakMap instance), you can open one with:

	SMLoader open!

!SMLoader classSide methodsFor: 'parts bin' stamp: 'gk 4/5/2005 21:23'!
descriptionForPartsBin
	^self partName: 'Package Loader'
		categories: #(Tools)
		documentation: 'SqueakMap UI'
! !

!SMLoader classSide methodsFor: 'initialization' stamp: 'gk 5/5/2006 02:05'!
initialize
	"Hook us up in the world menu."
	
	"self initialize"

	self registerInFlapsRegistry.
	(Preferences windowColorFor: #SMLoader) = Color white "not set"
		ifTrue: [ Preferences setWindowColorFor: #SMLoader to: (Color colorFrom: self windowColorSpecification brightColor) ].
	 (TheWorldMenu respondsTo: #registerOpenCommand:)
         ifTrue: [
		TheWorldMenu registerOpenCommand: {'SqueakMap Package Loader'. {self. #open}}.
		TheWorldMenu unregisterOpenCommand: 'Package Loader'].
	DefaultFilters := OrderedCollection new.
	DefaultCategoriesToFilterIds := OrderedCollection new
! !

!SMLoader classSide methodsFor: 'new-morph participation' stamp: 'gk 4/5/2005 21:21'!
initializedInstance
	^self new createWindow extent: 400@400! !

!SMLoader classSide methodsFor: 'instance creation' stamp: 'gk 6/21/2005 10:47'!
new
	"Create a SqueakMap loader on the default map."

	^self newOn: SMSqueakMap default! !

!SMLoader classSide methodsFor: 'instance creation' stamp: 'gk 6/21/2005 10:47'!
newOn: aMap
	"Create a SqueakMap loader on given map."

	^super new on: aMap; yourself! !

!SMLoader classSide methodsFor: 'new-morph participation' stamp: 'gk 4/5/2005 21:21'!
newStandAlone
	^self new createWindow! !

!SMLoader classSide methodsFor: 'instance creation' stamp: 'gk 6/21/2005 09:28'!
open
	"Create and open a SqueakMap Loader."
	
	"SMLoader open"

	^self new openAsMorph! !

!SMLoader classSide methodsFor: 'instance creation' stamp: 'gk 4/5/2005 21:18'!
openOn: aSqueakMap
	"Create and open a SqueakMap Loader on a given map."

	"self openOn: SqueakMap default"

	^(self newOn: aSqueakMap) openAsMorph! !

!SMLoader classSide methodsFor: 'new-morph participation' stamp: 'gk 6/21/2005 11:06'!
prototypicalToolWindow
	^self new createWindow; applyModelExtent; yourself! !

!SMLoader classSide methodsFor: 'new-morph participation' stamp: 'gk 6/21/2005 10:43'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry."

	self environment
		at: #Flaps
		ifPresent: [:cl | (cl respondsTo: #registerQuad:forFlapNamed:)
				ifTrue: [cl registerQuad: #(#SMLoader #prototypicalToolWindow 'Package Loader' 'The SqueakMap Package Loader' ) forFlapNamed: 'Tools']]! !

!SMLoader classSide methodsFor: 'initialization' stamp: 'gk 4/5/2005 22:19'!
unload
	(TheWorldMenu respondsTo: #registerOpenCommand:) ifTrue: 
		[TheWorldMenu unregisterOpenCommand: 'SqueakMap Package Loader'].
	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!SMLoader classSide methodsFor: 'window color' stamp: 'gk 6/21/2005 09:44'!
windowColorSpecification
	"Answer a WindowColorSpec object that declares my preference."

	^WindowColorSpec
		classSymbol: self name
		wording: 'Package Loader'
		brightColor: Color yellow muchLighter duller
		pastelColor: Color yellow veryMuchLighter duller
		helpMessage: 'The SqueakMap Package Loader'! !

!SMLoader methodsFor: 'menus' stamp: 'gk 7/13/2004 17:20'!
addFiltersToMenu: aMenu
	| filterSymbol help |
	self filterSpecs do: [:filterArray | 
		filterSymbol := filterArray second.
		help := filterArray third.
		aMenu addUpdating: #showFilterString: target: self selector: #toggleFilterState: argumentList: (Array with: filterSymbol).
		aMenu balloonTextForLastItem: help].
	aMenu addLine;
		addList: #(('uncheck all filters' uncheckFilters 'unchecks all filters so that all packages are listed'))
	! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 5/5/2006 02:05'!
addPackagesTo: window at: fractions plus: verticalOffset
	"Add the list for packages, and answer the verticalOffset plus the height added"

	| divider listMorph |
	listMorph := self buildMorphicPackagesList.
	listMorph borderWidth: 0.
	divider := BorderedSubpaneDividerMorph forBottomEdge.
	Preferences alternativeWindowLook ifTrue:[
		divider extent: 4@4; color: Color transparent; borderColor: #raised; borderWidth: 2.
	].
	window 
		addMorph: listMorph
		
	! !

!SMLoader methodsFor: 'actions' stamp: 'gk 7/12/2004 14:42'!
addSelectedCategoryAsFilter
	"Add a new filter that filters on the currently selected category.
	Make it enabled as default."

	categoriesToFilterIds add: self selectedCategory id! !

!SMLoader methodsFor: 'actions' stamp: 'gk 9/25/2006 23:14'!
askToLoadUpdates
	"Check how old the map is and ask to update it
	if it is older than 10 days or if there is no map on disk."

	| available |
	available := model isCheckpointAvailable.
	(available not or: [
		(Date today subtractDate: (Date fromSeconds:
			(model directory directoryEntryFor: model lastCheckpointFilename)
				modificationTime)) > 3])
		ifTrue: [
			(self confirm: 
				(available ifTrue: ['The map on disk is more than 10 days old,
update it from the Internet?'] ifFalse: ['There is no map on disk,
fetch it from the Internet?']))
				ifTrue: [self loadUpdates]]! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 9/25/2006 23:13'!
browseCacheDirectory
	"Open a FileList2 on the directory for the package or release."

	| item dir win |
	item := self selectedPackageOrRelease.
	item ifNil: [^nil].
	dir := item isPackage
				ifTrue: [model cache directoryForPackage: item]
				ifFalse: [model cache directoryForPackageRelease: item].
	win := FileList2 morphicViewOnDirectory: dir. " withLabel: item name, ' cache directory'."
	win openInWorld
! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 6/21/2005 15:04'!
buildMorphicCategoriesList
	"Create the hierarchical list holding the category tree."

	| list |
	list := (SimpleHierarchicalListMorph 
		on: self
		list: #categoryWrapperList
		selected: #selectedCategoryWrapper
		changeSelected: #selectedCategoryWrapper:
		menu: #categoriesMenu:
		keystroke: nil)
		autoDeselect: true;
		enableDrag: false;
		enableDrop: true;
		yourself.
	list setBalloonText: 'The categories are structured in a tree. Packages and package releases belong to several categories.
You can add one or more categories as filters and enable them in the menu.'.
"	list scroller submorphs do:[:each| list expandAll: each]."
	list adjustSubmorphPositions.
	^list! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 7/11/2004 03:04'!
buildMorphicPackagesList
	"Create the hierarchical list holding the packages and releases."

	^(SimpleHierarchicalListMorph 
		on: self
		list: #packageWrapperList
		selected: #selectedItemWrapper
		changeSelected: #selectedItemWrapper:
		menu: #packagesMenu:
		keystroke: nil)
		autoDeselect: false;
		enableDrag: false;
		enableDrop: true;
		setBalloonText: 'Here all packages with their releases are listed that should be displayed according the current filter.';
		yourself! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 5/5/2006 02:05'!
buildPackagePane
	"Create the text area to the right in the loader."

	| ptm |
	ptm := PluggableTextMorph 
		on: self 
		text: #contents
		accept: nil
		readSelection: nil "#packageSelection "
		menu: nil.
	ptm setBalloonText: 'This is where the selected package or package release is displayed.'.
	ptm lock.
	^ptm! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 5/5/2006 02:05'!
buildSearchPane
	| typeInView |
	typeInView := PluggableTextMorph on: self 
		text: nil accept: #findPackage:notifying:
		readSelection: nil menu: nil.
	typeInView setBalloonText:'To find a package type in a fragment of its name and hit return'.
	typeInView acceptOnCR: true.
	(typeInView respondsTo: #hideScrollBarsIndefinitely) ifTrue: [
		typeInView hideScrollBarsIndefinitely]
	ifFalse: [typeInView hideScrollBarIndefinitely].
	^typeInView! !

!SMLoader methodsFor: 'actions' stamp: 'gk 5/5/2006 02:05'!
cachePackageReleaseAndOfferToCopy
	"Cache package release, then offer to copy it somewhere.
	Answer the chosen file's location after copy,
	or the cache location if no directory was chosen."

	| release installer newDir newName newFile oldFile oldName |
	release := self selectedPackageOrRelease.
	release isPackageRelease ifFalse: [ self error: 'Should be a package release!!'].
	installer := SMInstaller forPackageRelease: release.
	[Cursor wait showWhile: [installer cache]] on: Error do: [:ex |
		| msg |
		msg := ex messageText ifNil: [ex asString].
		self informException: ex msg: ('Error occurred during download:\', msg, '\') withCRs.
		^nil ].
	installer isCached ifFalse: [self inform: 'Download failed, see transcript for details'. ^nil].
	oldName := installer fullFileName.
	newDir := FileList2 modalFolderSelector: installer directory.
	newDir ifNil: [ ^oldName ].
	newDir = installer directory ifTrue: [ ^oldName ].
	newName := newDir fullNameFor: installer fileName.
	newFile := FileStream newFileNamed: newName.
	newFile ifNil: [ ^oldName ].
	newFile binary.
	oldFile := FileStream readOnlyFileNamed: oldName.
	oldFile ifNil: [ ^nil ].
	oldFile binary.
	[[ newDir copyFile: oldFile toFile: newFile ] ensure: [ oldFile close. newFile close ]] on: Error do: [ :ex | ^oldName ].
	^newName! !

!SMLoader methodsFor: 'menus' stamp: 'gk 7/10/2004 03:58'!
categoriesMenu: aMenu 
	"Answer the categories-list menu."

	self selectedCategory 
		ifNotNil: [aMenu addList: self categorySpecificOptions; addLine].
	aMenu addList: self generalOptions.
	self addFiltersToMenu: aMenu.
	^aMenu! !

!SMLoader methodsFor: 'menus' stamp: 'md 7/13/2006 10:47'!
categorySpecificOptions
	| choices |
	choices := OrderedCollection new.
	(categoriesToFilterIds includes: self selectedCategory id)
		ifTrue: [
			choices add: #('remove filter' #removeSelectedCategoryAsFilter 'Remove the filter for the selected category.')]
		ifFalse: [
			choices add: #('add as filter' #addSelectedCategoryAsFilter 'Add the selected category as a filter so that only packages in that category are shown.')].
	categoriesToFilterIds isEmpty ifFalse: [
		choices add: #('remove all category filters' #removeCategoryFilters 'Remove all category filters.')].
	^ choices! !

!SMLoader methodsFor: 'lists' stamp: 'gk 9/25/2006 23:12'!
categoryWrapperList
	"Create the wrapper list for the hierarchical list.
	We sort the categories by name but ensure that 'Squeak versions'
	is first if it exists."
	 
	| list first |
	list := (((model categories select:[:each | each parent == nil]) asArray 
		sort:[:c1 :c2 | c1 name <= c2 name])).
	first := list detect:[:any | any name = 'Squeak versions'] ifNone:[nil].
	first ifNotNil:[
		list := list copyWithout: first.
		list := {first}, list].
	^list collect:[:cat | SMCategoryWrapper with: cat model: self].! !

!SMLoader methodsFor: 'accessing' stamp: 'gk 7/10/2004 15:45'!
changeFilters: anObject 
	"Update my selection."

	| oldItem index |
	oldItem := self selectedPackageOrRelease.
	filters := anObject.
	self packagesListIndex: ((index := self packageList indexOf: oldItem) 
				ifNil: [0]
				ifNotNil: [index]).
	self noteChanged! !

!SMLoader methodsFor: 'private' stamp: 'gk 5/5/2006 02:05'!
contents
	| packageOrRelease |
	packageOrRelease := self selectedPackageOrRelease.
	^packageOrRelease
		ifNil: ['<No package selected>']
		ifNotNil: [packageOrRelease fullDescription]
! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 7/13/2004 16:14'!
createWindow
	"Create the package loader window."
	
	self addMorph: (self buildSearchPane borderWidth: 0)
		frame: (0.0 @ 0.0 corner: 0.3 @ 0.07).
	self addMorph: (self buildMorphicPackagesList borderWidth: 0)
		frame: (0.0 @ 0.07 corner: 0.3 @ 0.6).
	self addMorph: (self buildMorphicCategoriesList borderWidth: 0)
		frame: (0.0 @ 0.6 corner: 0.3 @ 1.0).
	self addMorph: (self buildPackagePane borderWidth: 0)
		frame: (0.3 @ 0.0 corner: 1.0 @ 1.0).
	self on: #mouseEnter send: #paneTransition: to: self.
	self on: #mouseLeave send: #paneTransition: to: self! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 7/12/2004 11:14'!
defaultButtonPaneHeight
	"Answer the user's preferred default height for new button panes."

	^ Preferences parameterAt: #defaultButtonPaneHeight ifAbsentPut: [25]! !

!SMLoader methodsFor: 'actions' stamp: 'gk 5/5/2006 02:05'!
downloadPackageRelease
	"Force a download of the selected package release into the cache."

	| release |
	release := self selectedPackageOrRelease.
	release isPackageRelease ifFalse: [ self error: 'Should be a package release!!'].
	[Cursor wait showWhile: [
		(SMInstaller forPackageRelease: release) download]
	] on: Error do: [:ex |
		| msg | 
		msg := ex messageText ifNil: [ex asString].
		self informException: ex msg: ('Error occurred during download:\', msg, '\') withCRs]! !

!SMLoader methodsFor: 'actions' stamp: 'gk 5/5/2006 02:05'!
emailPackageMaintainers
	"Send mail to package owner and co-maintainers."

	| item package toAddresses |
	item := self selectedPackageOrRelease.
	package := item isPackageRelease ifTrue: [item package] ifFalse: [item].

	"(this logic should be moved to MailMessage as soon as it can handle 
multiple To: addresses)"
	toAddresses := '<', package owner email, '>'.
	package maintainers ifNotNil: [
		package maintainers do: [:maintainer |
			toAddresses := toAddresses, ', <', maintainer email, '>']].

	SMUtilities sendMailTo: toAddresses regardingPackageRelease: item! !

!SMLoader methodsFor: 'filter utilities' stamp: 'gk 7/10/2004 15:45'!
filterAdd: anObject

	self changeFilters: (self filters copyWith: anObject)
! !

!SMLoader methodsFor: 'filters' stamp: 'dvf 10/25/2002 11:29'!
filterAutoInstall
	^[:package | package isInstallable]! !

!SMLoader methodsFor: 'filters' stamp: 'gk 7/13/2004 15:28'!
filterAvailable
	
	^[:package | package isAvailable]! !

!SMLoader methodsFor: 'filters' stamp: 'dvf 10/25/2002 17:08'!
filterInstalled
	^[:package | package isInstalled]! !

!SMLoader methodsFor: 'filters' stamp: 'dvf 10/25/2002 17:08'!
filterNotInstalledYet
	^[:package | package isInstalled not]! !

!SMLoader methodsFor: 'filters' stamp: 'dvf 10/25/2002 17:07'!
filterNotUptoDate
	
	^[:package | package isAvailable]! !

!SMLoader methodsFor: 'filters' stamp: 'gk 1/28/2004 23:42'!
filterPublished
	^[:package | package isPublished]! !

!SMLoader methodsFor: 'filter utilities' stamp: 'gk 7/10/2004 15:45'!
filterRemove: anObject

	self changeFilters: (self filters copyWithout: anObject)
! !

!SMLoader methodsFor: 'filters' stamp: 'gk 7/13/2004 15:28'!
filterSafelyAvailable
	
	^[:package | package isSafelyAvailable]! !

!SMLoader methodsFor: 'filter utilities' stamp: 'gk 9/25/2006 23:13'!
filterSpecs
	"Return a specification for the filter menu. Is called each time."

	| specs |
	specs := #(
	#('display only auto-installable packages' #filterAutoInstall 'display only packages that can be installed automatically')
	#('display only new available packages' #filterAvailable 'display only packages that are not installed or that have newer releases available.')
	#('display only new safely available packages' #filterSafelyAvailable 'display only packages that are not installed or that have newer releases available that are safe to install, meaning that they are published and meant for the current version of Squeak.')
	#('display only installed packages' #filterInstalled 'display only packages that are installed.')
	#('display only published packages' #filterPublished 'display only packages that have at least one published release.'))
		asOrderedCollection.
	categoriesToFilterIds do: [:catId |
		specs add: {'display only packages in ', (model object: catId) name. catId. 'display only packages that are in the category.'}].
	^ specs! !

!SMLoader methodsFor: 'filters' stamp: 'gk 7/11/2004 03:02'!
filterVersion
	"Ignore spaces in the version string, they're sometimes spurious.

	Not used anymore."

	^[:package | package categories anySatisfy:  
		[:cat | (cat name, '*') match: (Smalltalk version copyWithout: $ ) ]]! !

!SMLoader methodsFor: 'filter utilities' stamp: 'gk 7/10/2004 15:47'!
filters
	^filters! !

!SMLoader methodsFor: 'actions' stamp: 'gk 7/10/2004 16:04'!
findPackage: aString notifying: aView 
	"Search and select a package with the given (sub) string"

	| index list match |
	match := aString asString asLowercase.
	index := self packagesListIndex.
	list := self packageNameList.
	list isEmpty ifTrue: [^self].
	index + 1 to: list size
		do: 
			[:i | 
			((list at: i) asLowercase includesSubString: match) 
				ifTrue: [^self packagesListIndex: i]].
	"wrap around"
	1 to: index
		do: 
			[:i | 
			((list at: i) asLowercase includesSubString: match) 
				ifTrue: [^self packagesListIndex: i]].
	self inform: 'No package matching ' , aString asString! !

!SMLoader methodsFor: 'menus' stamp: 'gk 7/14/2004 16:30'!
generalOptions
	^#(#('help' #help) 
		#('update map from the net' loadUpdates)
		#('upgrade all installed packages' upgradeInstalledPackagesNoConfirm)
		#('upgrade all installed packages confirming each' upgradeInstalledPackagesConfirm)
		#('put list in paste buffer' listInPasteBuffer)
		#('save filters as default' saveFiltersAsDefault)
		#- )

! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 5/5/2006 02:05'!
help
	"Present help text. If there is a web server available, offer to open it.
	Use the WebBrowser registry if possible, or Scamper if available."
	| message browserClass |
	message := 'Welcome to the SqueakMap package loader. 
The names of packages are followed by (installed version -> latest version). 
If there is no arrow, your installed version of the package is the latest.
The checkbox menu items at the bottom let you modify which packages 
you''ll see. Take a look at them - only some packages are shown initially. 
The options available for a package depend on how it was packaged. 
If you like a package or have comments on it, please contact
the author or the squeak mailing list.'.

	browserClass := Smalltalk at: #WebBrowser ifPresent: [ :registry | registry default ].
	browserClass := browserClass ifNil: [ Smalltalk at: #Scamper ifAbsent: [ ^self inform: message ]].

	(self confirm: message, '
Would you like to view more detailed help on the SqueakMap swiki page?') 
	ifTrue: [ browserClass openOnUrl: 'http://minnow.cc.gatech.edu/squeak/2726' asUrl]! !

!SMLoader methodsFor: 'private' stamp: 'gk 7/10/2004 04:04'!
informException: ex msg: msg 
	"Tell the user that an error has occurred.
	Offer to open debug notifier."

	(self confirm: msg, 'Would you like to open a debugger?')
		ifTrue: [ex pass]! !

!SMLoader methodsFor: 'actions' stamp: 'gk 5/5/2006 02:05'!
installPackageRelease
	"Install selected package or release.
	The cache is used."

	| item release |
	item := self selectedPackageOrRelease.
	item isPackageRelease
		ifTrue: [
			(item isPublished or: [self confirm: 'Selected release is not published yet, install anyway?'])
				ifTrue: [^self installPackageRelease: item]]
		ifFalse: [
			release := item lastPublishedReleaseForCurrentSystemVersion.
			release ifNil: [
				(self confirm: 'The package has no published release for your Squeak version, try releases for any Squeak version?')
					ifTrue: [
						release := item lastPublishedRelease.
						release ifNil: [
							(self confirm: 'The package has no published release at all, take the latest of the unpublished releases?')
								ifTrue: [release := item lastRelease]]]].
			release ifNotNil: [^self installPackageRelease: release]]! !

!SMLoader methodsFor: 'private' stamp: 'gk 9/26/2006 00:41'!
installPackageRelease: aRelease
	"Install a package release. The cache is used."

	| myRelease |
	aRelease isCompatibleWithCurrentSystemVersion ifFalse:
		[(self confirm:
'The package you are about to install is not listed as
being compatible with your image version (', SystemVersion current majorMinorVersion, '),
so the package may not work properly.
Do you still want to proceed with the install?')
			ifFalse: [^ self]].
	myRelease := self installedReleaseOfMe.
	[Cursor wait showWhile: [
		(SMInstaller forPackageRelease: aRelease) install.
		myRelease = self installedReleaseOfMe
					ifFalse: [self reOpen]
					ifTrue: [self noteChanged]]
	] on: Error do: [:ex |
		| msg |
		msg := ex messageText ifNil:[ex asString].
		self informException: ex msg: ('Error occurred during install:\', msg, '\') withCRs].! !

!SMLoader methodsFor: 'private' stamp: 'gk 9/26/2006 00:25'!
installedReleaseOfMe
	"Return the release of the installed package loader."

	^SMSqueakMap default installedReleaseOf: (SMSqueakMap default packageWithId: '941c0108-4039-4071-9863-a8d7d2b3d4a3').! !

!SMLoader methodsFor: 'filter utilities' stamp: 'dvf 10/25/2002 11:27'!
labelForFilter: aFilterSymbol 
	^(self filterSpecs detect: [:fs | fs second = aFilterSymbol]) first! !

!SMLoader methodsFor: 'actions' stamp: 'gk 11/18/2003 02:19'!
listInPasteBuffer
	"Useful when talking with people etc.
	Uses the map to produce a nice String."

	Clipboard clipboardText:
		(String streamContents: [:s |
			packagesList do: [:p |
				s nextPutAll: p nameWithVersionLabel; cr ]]) asText! !

!SMLoader methodsFor: 'actions' stamp: 'gk 9/25/2006 23:12'!
loadUpdates
	[Cursor wait showWhile: [
		model loadUpdates.
		self noteChanged ]
	] on: Error do: [:ex |
		self informException: ex msg: ('Error occurred when updating map:\', ex messageText, '\') withCRs]! !

!SMLoader methodsFor: 'private' stamp: 'gk 9/26/2006 01:02'!
noteChanged
	filters ifNil: [^self reOpen].
	model ifNotNil: [
		packagesList := nil.
		selectedCategoryWrapper := nil.
		self changed: #categoryWrapperList.
		self changed: #packageWrapperList.
		self changed: #packagesListIndex.	"update my selection"
		self contentsChanged]! !

!SMLoader methodsFor: 'initialization' stamp: 'gk 9/25/2006 23:12'!
on: aSqueakMap 
	"Initialize instance."

	model := aSqueakMap.
	model synchWithDisk.
	filters := DefaultFilters copy.
	categoriesToFilterIds := DefaultCategoriesToFilterIds copy.
	self askToLoadUpdates! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 4/5/2005 21:17'!
openAsMorph
	"Open the loader as a Morphic window."
	"SMLoader new openAsMorph"
	
	^self createWindow openInWorld! !

!SMLoader methodsFor: 'filter utilities' stamp: 'gk 7/11/2004 22:29'!
package: aPackage filteredByCategory: aCategory
	"Answer true if the package should be shown
	if we filter on <aCategory>. It should be shown
	if itself or any of its releases has the category."

	| releases |
	releases := aPackage releases.
	^(aPackage hasCategoryOrSubCategoryOf: aCategory) or: [
			releases anySatisfy: [:rel |
				rel hasCategoryOrSubCategoryOf: aCategory]]! !

!SMLoader methodsFor: 'lists' stamp: 'gk 9/25/2006 23:13'!
packageList
	"Return a list of the SMPackages that should be visible
	by applying all the filters. Also filter based on the currently
	selected category - if any."

	| list selectedCategory |
	list := packagesList ifNil: [
			packagesList := self packages select: [:p | 
				filters allSatisfy: [:currFilter |
					currFilter isSymbol
						ifTrue: [(self perform: currFilter) value: p]
						ifFalse: [
						self package: p
							filteredByCategory: (model object: currFilter)]]]].
	selectedCategoryWrapper ifNil:[self updateLabel: list. ^list].
	selectedCategory := selectedCategoryWrapper category.
	list := list select: [:each | self package: each filteredByCategory: selectedCategory].
	self updateLabel: list.
	^list! !

!SMLoader methodsFor: 'lists' stamp: 'dvf 9/21/2003 16:36'!
packageNameList
	^self packageWrapperList collect: [:e | e withoutListWrapper name]! !

!SMLoader methodsFor: 'menus' stamp: 'gk 7/9/2004 19:44'!
packageSpecificOptions
	| choices packageOrRelease |
	packageOrRelease := self selectedPackageOrRelease.
	choices := OrderedCollection new.
	packageOrRelease isInstallable ifTrue: [
		choices add: #('install' #installPackageRelease 'Install selected package or release, first downloading into the cache if needed.')].	
	(packageOrRelease isDownloadable and: [packageOrRelease isCached]) ifTrue: [
		choices add: #('browse cache' #browseCacheDirectory 'Browse cache directory of selected package or package release.')].

	(packageOrRelease isPackageRelease and: [packageOrRelease isDownloadable]) ifTrue: [
		choices add: #('copy from cache' #cachePackageReleaseAndOfferToCopy 'Download selected release into cache first if needed, and then offer to copy it somewhere else.' ).
		choices add: #('force download into cache' #downloadPackageRelease 'Force a download of the selected release into the cache.' )].
	choices add: #('email package maintainers' emailPackageMaintainers 'Open an editor to send an email to the owner and co-maintainers of this package.').
	^ choices! !

!SMLoader methodsFor: 'lists' stamp: 'gk 7/9/2004 01:36'!
packageWrapperList
	"Return the list with each element wrapped so that it
	can be used in a SimpleHierarchicalListMorph."

	^self packageList collect: [:e | SMPackageWrapper with: e]! !

!SMLoader methodsFor: 'private' stamp: 'gk 9/25/2006 23:12'!
packages
	"We request the packages as sorted by name by default."

	^model packagesByName asArray
! !

!SMLoader methodsFor: 'accessing' stamp: 'dvf 9/21/2003 16:34'!
packagesListIndex
	^self packageWrapperList indexOf: self selectedItemWrapper! !

!SMLoader methodsFor: 'accessing' stamp: 'dvf 9/21/2003 16:53'!
packagesListIndex: anObject
	self selectedItemWrapper: (anObject = 0 ifTrue:[nil] ifFalse: [(self packageWrapperList at: anObject)])
! !

!SMLoader methodsFor: 'menus' stamp: 'gk 7/9/2004 03:04'!
packagesMenu: aMenu 
	"Answer the packages-list menu."

	self selectedPackageOrRelease 
		ifNotNil: [aMenu addList: self packageSpecificOptions; addLine].
	aMenu addList: self generalOptions.
	self addFiltersToMenu: aMenu.
	^aMenu! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 4/5/2005 21:43'!
paneColorOld
	^ Color yellow muchLighter duller! !

!SMLoader methodsFor: 'gui building' stamp: 'gk 4/5/2005 21:43'!
perform: selector orSendTo: otherTarget 
	"Selector was just chosen from a menu by a user. If can respond, then  
	perform it on myself. If not, send it to otherTarget, presumably the  
	editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ super perform: selector orSendTo: otherTarget]! !

!SMLoader methodsFor: 'private' stamp: 'gk 7/11/2004 04:06'!
reOpen
	"Close this package loader, probably because it has been updated,
	and open a new one."

	self inform: 'This package loader has been upgraded and will be closed and reopened to avoid strange side effects.'.
	self delete.
	SMLoader open! !

!SMLoader methodsFor: 'actions' stamp: 'gk 7/12/2004 14:42'!
removeCategoryFilters
	"Remove all category filters."

	categoriesToFilterIds := OrderedCollection new! !

!SMLoader methodsFor: 'actions' stamp: 'gk 7/12/2004 14:42'!
removeSelectedCategoryAsFilter
	"Remove the filter that filters on the currently selected category."

	categoriesToFilterIds remove: self selectedCategory id! !

!SMLoader methodsFor: 'actions' stamp: 'gk 7/12/2004 14:42'!
saveFiltersAsDefault
	"Save the current filters as default so that they
	are selected the next time the loader is opened."

	DefaultFilters := filters copy.
	DefaultCategoriesToFilterIds := categoriesToFilterIds copy! !

!SMLoader methodsFor: 'accessing' stamp: 'gk 7/10/2004 03:58'!
selectedCategory
	"Return selected category."

	^(self selectedCategoryWrapper isNil)
		ifFalse: [self selectedCategoryWrapper withoutListWrapper]! !

!SMLoader methodsFor: 'accessing' stamp: 'ar 2/9/2004 02:14'!
selectedCategoryWrapper
	^selectedCategoryWrapper! !

!SMLoader methodsFor: 'accessing' stamp: 'ar 2/9/2004 02:53'!
selectedCategoryWrapper: aWrapper
	selectedCategoryWrapper := aWrapper.
	self selectedItemWrapper: nil.
	self changed: #selectedCategoryWrapper.
	self changed: #packageWrapperList.! !

!SMLoader methodsFor: 'accessing' stamp: 'dvf 9/21/2003 15:04'!
selectedItemWrapper
	^selectedItemWrapper! !

!SMLoader methodsFor: 'accessing' stamp: 'gk 5/5/2006 02:05'!
selectedItemWrapper: aWrapper
	selectedItemWrapper := aWrapper.
	self changed: #selectedItemWrapper.
	self contentsChanged! !

!SMLoader methodsFor: 'private' stamp: 'gk 11/16/2003 20:12'!
selectedPackageOrRelease
	"Return selected package or package release."

	^(self selectedItemWrapper isNil) ifFalse: [self selectedItemWrapper withoutListWrapper]! !

!SMLoader methodsFor: 'filter utilities' stamp: 'dvf 10/25/2002 14:48'!
showFilterString: aFilterSymbol 
	^(self stateForFilter: aFilterSymbol), (self labelForFilter: aFilterSymbol)! !

!SMLoader methodsFor: 'filter utilities' stamp: 'gk 7/10/2004 15:45'!
stateForFilter: aFilterSymbol 
	^(self filters includes: aFilterSymbol) ifTrue: ['<yes>'] ifFalse: ['<no>']

! !

!SMLoader methodsFor: 'filter utilities' stamp: 'gk 7/10/2004 15:46'!
toggleFilterState: aFilterSymbol 

	^(self filters includes: (aFilterSymbol)) 
		ifTrue: [self filterRemove: aFilterSymbol]
		ifFalse: [self filterAdd: aFilterSymbol]! !

!SMLoader methodsFor: 'actions' stamp: 'gk 7/13/2004 16:05'!
uncheckFilters
	"Uncheck all filters."
	
	filters := OrderedCollection new.
	self noteChanged! !

!SMLoader methodsFor: 'lists' stamp: 'gk 9/25/2006 23:13'!
updateLabel: packagesShown
	"Update the label of the window."

	self setLabel: 'SqueakMap Package Loader (', packagesShown size printString,
			'/', model packages size printString, ')'! !

!SMLoader methodsFor: 'actions' stamp: 'gk 9/26/2006 00:40'!
upgradeInstalledPackages
	"Tries to upgrade all installed packages to the latest published release for this
	version of Squeak. So this is a conservative approach."

	| installed old myRelease toUpgrade info |
	installed := model installedPackages.
	old := model oldPackages.
	old isEmpty ifTrue: [
			^self inform: 'All ', installed size printString, ' installed packages are up to date.'].
	toUpgrade := model upgradeableAndOldPackages.
	toUpgrade isEmpty ifTrue: [
			^self inform: 'None of the ', old size printString, ' old packages of the ', installed size printString, ' installed can be automatically upgraded. You need to upgrade them manually.'].
	old size < toUpgrade size ifTrue: [
		info := 'Of the ', old size printString, ' old packages only ', toUpgrade size printString, ' can be upgraded.
The following packages will not be upgraded:
',  (String streamContents: [:s | (old removeAll: toUpgrade; yourself)
	do: [:p | s nextPutAll: p nameWithVersionLabel; cr]])]
		ifFalse: [info := 'All old packages upgradeable.'].
	(self confirm: info, '
About to upgrade the following packages:
', (String streamContents: [:s | toUpgrade do: [:p | s nextPutAll: p nameWithVersionLabel; cr]]), 'Proceed?') ifTrue: [
			myRelease := self installedReleaseOfMe.
			[Cursor wait showWhile: [
				model upgradeOldPackages.
				self inform: toUpgrade size printString, ' packages successfully upgraded.'.
				myRelease = self installedReleaseOfMe
					ifFalse: [self reOpen]
					ifTrue: [self noteChanged]]
			] on: Error do: [:ex |
				self informException: ex msg: ('Error occurred when upgrading old packages:\', ex messageText, '\') withCRs]]! !

!SMLoader methodsFor: 'actions' stamp: 'gk 7/14/2004 16:29'!
upgradeInstalledPackagesConfirm
	"Tries to upgrade all installed packages to the latest published release for this
	version of Squeak. Confirms on each upgrade."

	^ self upgradeInstalledPackagesConfirm: true! !

!SMLoader methodsFor: 'private' stamp: 'gk 9/26/2006 00:40'!
upgradeInstalledPackagesConfirm: confirmEach
	"Tries to upgrade all installed packages to the latest published release for this
	version of Squeak. If confirmEach is true we ask for every upgrade."

	| installed old myRelease toUpgrade info |
	installed := model installedPackages.
	old := model oldPackages.
	old isEmpty ifTrue: [
			^self inform: 'All ', installed size printString, ' installed packages are up to date.'].
	toUpgrade := model upgradeableAndOldPackages.
	toUpgrade isEmpty ifTrue: [
			^self inform: 'None of the ', old size printString, ' old packages of the ', installed size printString, ' installed can be automatically upgraded. You need to upgrade them manually.'].
	old size < toUpgrade size ifTrue: [
		info := 'Of the ', old size printString, ' old packages only ', toUpgrade size printString, ' can be upgraded.
The following packages will not be upgraded:
',  (String streamContents: [:s | (old removeAll: toUpgrade; yourself)
	do: [:p | s nextPutAll: p nameWithVersionLabel; cr]])]
		ifFalse: [info := 'All old packages upgradeable.'].
	(self confirm: info, '
About to upgrade the following packages:
', (String streamContents: [:s | toUpgrade do: [:p | s nextPutAll: p nameWithVersionLabel; cr]]), 'Proceed?') ifTrue: [
			myRelease := self installedReleaseOfMe.
			[Cursor wait showWhile: [
				confirmEach ifTrue: [
					model upgradeOldPackagesConfirmBlock: [:p |
						self confirm: 'Upgrade ', p installedRelease packageNameWithVersion, ' to ',
							(p lastPublishedReleaseForCurrentSystemVersionNewerThan: p installedRelease) listName, '?']]
						ifFalse: [model upgradeOldPackages].
				self inform: toUpgrade size printString, ' packages successfully processed.'.
				myRelease = self installedReleaseOfMe
					ifFalse: [self reOpen]
					ifTrue: [self noteChanged]]
			] on: Error do: [:ex |
				self informException: ex msg: ('Error occurred when upgrading old packages:\', ex messageText, '\') withCRs]]! !

!SMLoader methodsFor: 'actions' stamp: 'gk 7/14/2004 16:29'!
upgradeInstalledPackagesNoConfirm
	"Tries to upgrade all installed packages to the latest published release for this
	version of Squeak. No confirmation on each upgrade."

	^ self upgradeInstalledPackagesConfirm: false! !

ListItemWrapper subclass: #SMCategoryWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMLoader'!

!SMCategoryWrapper commentStamp: '<historical>' prior: 0!
This is a wrapper for showing the SqueakMap categories (SMCategory) using the SimpleHierarchicalListMorph in the SMLoader, see SMLoader>>categoryWrapperList.!

!SMCategoryWrapper methodsFor: 'as yet unclassified' stamp: 'ar 2/9/2004 02:13'!
= anObject
	^self withoutListWrapper = anObject withoutListWrapper! !

!SMCategoryWrapper methodsFor: 'as yet unclassified' stamp: 'ar 2/9/2004 02:19'!
asString
	^item name! !

!SMCategoryWrapper methodsFor: 'as yet unclassified' stamp: 'ar 2/9/2004 02:35'!
category
	^item! !

!SMCategoryWrapper methodsFor: 'as yet unclassified' stamp: 'gk 3/4/2004 10:18'!
contents
	"This is the message that returns the contents of this wrapper.
	We return a collection of wrappers around all the children of our model."

	^item subCategories collect: [:e | SMCategoryWrapper with: e]! !

!SMCategoryWrapper methodsFor: 'as yet unclassified' stamp: 'ar 2/9/2004 02:13'!
hash
	^self withoutListWrapper hash! !

ListItemWrapper subclass: #SMPackageReleaseWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMLoader'!

!SMPackageReleaseWrapper commentStamp: 'gk 6/21/2005 10:54' prior: 0!
This is a wrapper for showing the SqueakMap package releases (SMPackageRelease) using the SimpleHierarchicalListMorph in the SMLoader, see SMPackageWrapper>>contents.!

!SMPackageReleaseWrapper methodsFor: 'as yet unclassified' stamp: 'gk 7/13/2004 13:57'!
asString
	"Show installed releases with a trailing asterisk."
	
	^ item isInstalled
		ifTrue: [item smartVersion, ' *']
		ifFalse: [item smartVersion]! !

ListItemWrapper subclass: #SMPackageWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMLoader'!

!SMPackageWrapper commentStamp: '<historical>' prior: 0!
This is a wrapper for showing the SqueakMap packages (SMPackage) using the SimpleHierarchicalListMorph in the SMLoader, see SMLoader>>packageWrapperList.!

!SMPackageWrapper methodsFor: 'as yet unclassified' stamp: 'dvf 9/21/2003 16:25'!
= anObject
	^self withoutListWrapper = anObject withoutListWrapper! !

!SMPackageWrapper methodsFor: 'as yet unclassified' stamp: 'gk 7/13/2004 00:34'!
asString
	^item nameWithVersionLabel! !

!SMPackageWrapper methodsFor: 'as yet unclassified' stamp: 'dvf 10/14/2003 18:58'!
contents
	^item releases reversed collect: [:e | SMPackageReleaseWrapper with: e]! !

!SMPackageWrapper methodsFor: 'as yet unclassified' stamp: 'dvf 9/21/2003 16:25'!
hash
	^self withoutListWrapper hash! !

!SMPackageWrapper methodsFor: 'as yet unclassified' stamp: 'dvf 9/21/2003 16:22'!
printOn: aStream
	aStream nextPutAll: 'wrapper for: ', item printString! !

!FileList2 classSide methodsFor: '*smloader-override' stamp: 'btr 1/30/2004 00:56'!
morphicView
	^ self morphicViewOnDirectory: FileDirectory default! !

!FileList2 classSide methodsFor: '*smloader-extension' stamp: 'gk 5/5/2006 02:05'!
morphicViewOnDirectory: aFileDirectory
	| aFileList window fileListBottom midLine fileListTopOffset buttonPane |

	aFileList := self new directory: aFileDirectory.
	window := (SystemWindow labelled: aFileDirectory pathName) model: aFileList.

	fileListTopOffset := (TextStyle defaultFont pointSize * 2) + 14.
	fileListBottom := 0.4.
	midLine := 0.4.
	buttonPane := aFileList optionalButtonRow addMorph:
		(aFileList morphicPatternPane vResizing: #spaceFill; yourself).
	self addFullPanesTo: window from: {
		{buttonPane. 0@0 corner: 1@0. 0@0 corner: 0@fileListTopOffset}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@fileListBottom. 
					0@fileListTopOffset corner: 0@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@fileListBottom. 
					0@fileListTopOffset corner: 0@0}.
		{aFileList morphicFileContentsPane. 0@fileListBottom corner: 1@1. nil}.
	}.
	aFileList postOpen.
	^ window ! !
SMLoader initialize!
